<?php

declare(strict_types=1);

namespace Dedi\SyliusCustomerPricingPlugin\Grid\Filter;

use Dedi\SyliusCustomerPricingPlugin\Entity\ChannelPricing\Customer\CustomerChannelPricingInterface;
use Dedi\SyliusCustomerPricingPlugin\Entity\ChannelPricing\CustomerGroup\CustomerGroupChannelPricingInterface;
use Dedi\SyliusCustomerPricingPlugin\Entity\ChannelPricing\Organization\OrganizationChannelPricingInterface;
use Dedi\SyliusCustomerPricingPlugin\Enum\ChannelPricingType;
use Doctrine\ORM\EntityManagerInterface;
use Sylius\Component\Grid\Data\DataSourceInterface;
use Sylius\Component\Grid\Filtering\FilterInterface;

class ChannelPricingTypeFilter implements FilterInterface
{
    public function __construct(private readonly EntityManagerInterface $entityManager)
    {
    }

    public function apply(DataSourceInterface $dataSource, string $name, $data, array $options): void
    {
        if ('' === $data) {
            return;
        }

        switch ($data) {
            case 'all':
                break;
            case ChannelPricingType::CUSTOMER->value:
                $dataSource->restrict('o INSTANCE OF ' . $this->entityManager->getClassMetadata(CustomerChannelPricingInterface::class)->getName());

                break;
            case ChannelPricingType::ORGANIZATION->value:
                $dataSource->restrict('o INSTANCE OF ' . $this->entityManager->getClassMetadata(OrganizationChannelPricingInterface::class)->getName());

                break;
            case ChannelPricingType::CUSTOMER_GROUP->value:
                $dataSource->restrict('o INSTANCE OF ' . $this->entityManager->getClassMetadata(CustomerGroupChannelPricingInterface::class)->getName());

                break;
            case ChannelPricingType::DEFAULT->value:
                $dataSource->restrict('o NOT INSTANCE OF ' . $this->entityManager->getClassMetadata(CustomerChannelPricingInterface::class)->getName() . ' AND o NOT INSTANCE OF ' . $this->entityManager->getClassMetadata(OrganizationChannelPricingInterface::class)->getName() . ' AND o NOT INSTANCE OF ' . $this->entityManager->getClassMetadata(CustomerGroupChannelPricingInterface::class)->getName());

                break;
            default:
                throw new \InvalidArgumentException('Invalid channel pricing type');
        }
    }
}
